## Dev Requirements

- [Node.js](https://nodejs.org/en/) (LTS is recommended)
- [yarn](https://yarnpkg.com/getting-started/install)
  - We use `yarn` as package manager instead of `npm`
  - Installation: `sudo npm install --global yarn`
      - `npm` comes with Node.js


## Project Setup

```sh
yarn install
```

`yarn install` installs all dependencies of the package (defined by package.json) in the project directory.
The packages are installed only locally inside the `node_modules` folder.

### Compile and Hot-Reload for Development

```sh
yarn dev
```

#### Development with real backend

To use a real backend during `development` mode (`yarn dev`) you have to copy the `.env.development.local.example` to `.env.development.local`.

In that file you can control whether to use a real backend by setting `VITE_USE_REAL_BACKEND` to `true` or `false`.

To configure which remote backend and keyclock to use you must copy `public/config.json.example` to `public/config.json`
and replace `backendURL` and `ssoURI` with your system.

The keycloak host must be a FQDN, the ucs apache needs to now the hostname.
Your local machine must be able to resolve the FQDN of keycloak (`ssoURI`).
E.g. in /etc/hosts
```
10.200.41.160 ucs-sso-ng.ram.jb41.ucs.icu
```

You might also need to adjust the `redirect_url` in keycloak for the client.
You can use the `univention-create-keycloak-clients` script to edit that.
```sh
/usr/share/ucs-school-ui-groups-frontend/univention-create-keycloak-clients --admin-username admin --admin-pwdfile /etc/keycloak.secret --client-id "school-ui-groups-dev" --recreate-client
```

### Use local univention-veb version

If you want to use a local version of `@univention/univention-veb` without using the Gitlab npm registry
you can do:

In `univention-veb` repo
```sh
yarn build
yarn pack
```

Here
```sh
yarn remove @univention/univention-veb
yarn cache clean
yarn add file:../../univention-veb/univention-univention-veb-vx.x.x.tgz
```


### Type-Check, Compile and Minify for Production

```sh
yarn build
```

### Preview the Production build locally

You need to run `yarn build` beforehand manually
```sh
yarn preview
```


### Run Unit Tests with [Vitest](https://vitest.dev/)

```sh
yarn test:unit
```

### Run End-to-End Tests with [Cypress](https://www.cypress.io/)

#### Setup

Copy `cypress.env.example.json` to `cypress.env.json` and adjust the values as necessary.

The `cypress.env.example.json` works with the domain created by the
[Create RAM Environment in KVM](https://univention-dist-jenkins.k8s.knut.univention.de/job/UCSschool-5.0/job/RAM-environment/)
jenkins job.

Additionally create a school and classes in your test environment that
matches the `SCHOOL` and `SCHOOL_CLASSES` fields in `cypress.env.json`
or adjust the fields according to your test environment.

#### cypress.env.json explanation
```json5
{
  // Domain of the simplesaml and keycloak login backends
  // Used to clear the cookies to clear logged in users.
  "SIMPLESAML_DOMAIN": "ucs-sso.school.test",
  "KEYCLOAK_DOMAIN": "ucs-sso-ng.school.test",

  // Default user for the `cy.login()` command
  "DEFAULT_USER_NAME": "admin",
  "DEFAULT_USER_PASSWORD": "univention",

  // Existing school in the test environment that is
  // used as the context for the tests
  "SCHOOL": "school1",
  // School classes existing in the test environment for `SCHOOL`.
  // Needed for delete and edit test of school classes
  "SCHOOL_CLASSES": [/*...*/]
}
```

#### Run tests

```sh
yarn build
yarn test:e2e # or `yarn test:e2e:ci` for headless testing
```

### Type-Check

```sh
yarn type-check
```

### Lint with [ESLint](https://eslint.org/)

```sh
yarn lint
```

To automatically fix some problems you can run

```sh
yarn lint:fix
```

### These commands and potentially more can be found in the `scripts` block in [package.json](package.json)


## IDEs
### VSCode

[VSCode](https://code.visualstudio.com/) + [Volar](https://marketplace.visualstudio.com/items?itemName=Vue.volar) (and disable Vetur) + [TypeScript Vue Plugin (Volar)](https://marketplace.visualstudio.com/items?itemName=Vue.vscode-typescript-vue-plugin).

If the standalone TypeScript plugin doesn't feel fast enough to you, Volar has also implemented a [Take Over Mode](https://github.com/johnsoncodehk/volar/discussions/471#discussioncomment-1361669) that is more performant. You can enable it by the following steps:

1. Disable the built-in TypeScript Extension
   1) Run `Extensions: Show Built-in Extensions` from VSCode's command palette
   2) Find `TypeScript and JavaScript Language Features`, right click and select `Disable (Workspace)`
2. Reload the VSCode window by running `Developer: Reload Window` from the command palette.

### WebStorm

[WebStorm](https://www.jetbrains.com/de-de/webstorm/) + [Vue setup](https://www.jetbrains.com/help/webstorm/vue-js.html#ws_vue_js_before_you_start)
